@@ -193,7 +193,9 @@ wm_usage(char *progname)
    printf("\t-theme            <string> \n");
    printf("\t-use_titlebar     <yes|no>\n");
    printf("\t-use_cursor       <yes|no>\n");
+#ifndef USE_COMPOSITE
    printf("\t-use_lowlight     <yes|no>\n");
+#endif
    printf("\t-use_dialog_mode  <free|const|const-horiz>\n");
    printf("\t-use_desktop_mode <decorated|plain>\n");
    printf("\t-ping_handler     <string>\n");
@@ -382,11 +384,16 @@ wm_load_config (Wm   *w,
 	 }
      }   
 
+   /* 
+    *  Composite matchbox always uses lowlighting ... 
+    */
+#ifndef USE_COMPOSITE
    if (XrmGetResource (rDB, "matchbox.lowlight", "Matchbox.Lowlight",
 		       &type, &value) == True)
      {
        if (strncmp (value.addr, "yes", (int) value.size) == 0)
 	 {
+#endif
 	   dbg("%s() TURNING LOWLIGHT ON\n", __func__);
 	   w->config->dialog_shade = True;   
 
@@ -395,8 +402,10 @@ wm_load_config (Wm   *w,
 	   w->config->lowlight_params[1] = 0; 
 	   w->config->lowlight_params[2] = 0; 
 	   w->config->lowlight_params[3] = 100; 
+#ifndef USE_COMPOSITE
      	 }
      }
+#endif
 
    if (XrmGetResource (rDB, "matchbox.dialog", "Matchbox.Dialog",
 		       &type, &value) == True)
@@ -1109,10 +1118,13 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	    || c->wm->config->dialog_stratergy == WM_DIALOGS_STRATERGY_FREE)
        )
      {
-       xwc.width  = e->width;
-       xwc.height = e->height;
-       xwc.x = e->x;
-       xwc.y = e->y;
+       c->width  = xwc.width  = e->width;
+       c->height = xwc.height = e->height;
+       c->x      = xwc.x = e->x;
+       c->y      = xwc.y = e->y;
+
+       /* Make sure we get the damage before the move.. */
+       comp_engine_client_show(c->wm, c); 
 
        need_comp_update = True;
        
@@ -1901,24 +1913,42 @@ wm_get_offsets_size(Wm*     w,
 void
 wm_toggle_desktop(Wm *w)
 {
-   Client *p;
+  Client *p;
 
-   dbg("%s() called desktop flag is : %i \n", __func__, 
-       (w->flags & DESKTOP_RAISED_FLAG));
+  dbg("%s() called desktop flag is : %i \n", __func__, 
+      (w->flags & DESKTOP_RAISED_FLAG));
 
    if (!wm_get_desktop(w)) 
-     return;
-
-   if (w->flags & DESKTOP_DECOR_FLAG)
      {
-       w->flags ^= DESKTOP_RAISED_FLAG;	
-       wm_activate_client(wm_get_desktop(w));
+       dbg("%s() couldn't find desktop \n", __func__ );
        return;
      }
 
+   /* Toggle decorated desktop */
+   if (w->flags & DESKTOP_DECOR_FLAG)
+     {
+       if (w->flags & DESKTOP_RAISED_FLAG)
+	 {
+	   if (w->prev_main_client)
+	     wm_activate_client(w->prev_main_client);
+	 }
+       else
+	 {
+	   if (w->main_client)
+	     w->prev_main_client = w->main_client;
+	   else
+	     w->prev_main_client = NULL;
+
+	   wm_activate_client(wm_get_desktop(w));
+	   w->flags ^= DESKTOP_RAISED_FLAG;
+	 }
+       return;
+     }
+
+   /* toggle  */
    XGrabServer(w->dpy);
    if (w->flags & DESKTOP_RAISED_FLAG)
-   { 				/* Desktop is visible, raise everything */
+   { 				/* Desktop is visible, raise everything! */
      dbg("%s() Desktop is raised, hiding it\n", __func__);
      if (w->main_client != NULL && w->main_client->type != desktop)
        {
@@ -1955,7 +1985,7 @@ wm_toggle_desktop(Wm *w)
       START_CLIENT_LOOP(w,p)
 	{
 	  if ( p->type == toolbar) 
-	    { /* p->hide(p); */ XMapRaised(w->dpy, p->frame); }
+	    { XMapRaised(w->dpy, p->frame); }
 	  if ( p->type == dock && !(p->flags & CLIENT_DOCK_TITLEBAR)) 
 	    XMapRaised(w->dpy, p->window);
 	  if (p->type == dialog && p->trans == NULL && p->mapped)
@@ -1969,7 +1999,6 @@ wm_toggle_desktop(Wm *w)
 	w->flags ^= DESKTOP_RAISED_FLAG;	
    }
 
-
    ewmh_update(w);
    ewmh_set_active(w);
 
